import re
from django.conf.urls import url, patterns, include
from django.conf import settings
from django.contrib import admin
from django.views.generic import TemplateView
from django.template import add_to_builtins

add_to_builtins('avocado.templatetags.avocado_tags')

admin.autodiscover()

urlpatterns = patterns('',
    # Home/Splash/Index page
    url(r'^$', TemplateView.as_view(template_name='index.html'), name='index'),

    # Mount point for the Cilantro app. Since this is a single-page web app
    # all URLs will be routed relative to this endpoint.
    url(r'^workspace/', 'cilantro.views.app', name='cilantro'),

    # Other Cilantro endpoints such as the preferences/session API
    url(r'^', include('cilantro.urls')),

    # Serrano API endpoints
    url(r'^api/', include('serrano.urls')),

    # Administrative components
    url(r'^admin/', include(admin.site.urls)),
)

# In production, these two locations must be served up statically
urlpatterns += patterns('django.views.static',
    url(r'^{}(?P<path>.*)$'.format(re.escape(settings.MEDIA_URL.lstrip('/'))), 'serve', {
        'document_root': settings.MEDIA_ROOT
    }),
    url(r'^{}(?P<path>.*)$'.format(re.escape(settings.STATIC_URL.lstrip('/'))), 'serve', {
        'document_root': settings.STATIC_ROOT
    }),
)



from __future__ import print_function
import re
import os
import sys
import harvest
from fabric.api import local, hide
from fabric.colors import red, green
from harvest.decorators import cli, virtualenv
from harvest.utils import create_virtualenv, managepy_chmod

__doc__ = """\
Creates and sets up a new Harvest project.
"""

HARVEST_TEMPLATE_PATH = os.path.join(os.path.abspath(os.path.dirname(harvest.__file__)), 'template')
STARTPROJECT_ARGS = '--template {} -e py,ini,gitignore,in,conf,md,sample ' \
    '-n Makefile'.format(HARVEST_TEMPLATE_PATH)


def valid_name(name):
    if re.match(r'^[a-z_]\w*$', name, re.I) is not None:
        return True
    return False


@cli(description=__doc__)
def parser(options):
    project_name = options.project_name
    create_env = options.create_env
    allow_input = options.allow_input
    verbose = options.verbose

    if not valid_name(project_name):
        print(red("Error: The project name '{}' must be a valid Python "
            "identifier.".format(project_name)))
        sys.exit()

    # Ensure the name does not conflict with an existing Python module
    try:
        __import__(project_name)
        print(red("Error: The project name '{}' conflicts with an existing "
            "Python module. Please choose another name.".format(project_name)))
        sys.exit()
    except ImportError:
        pass

    hidden_output = []

    if verbose < 1:
        hidden_output.append('stdout')
    if verbose < 2:
        hidden_output.append('running')

    with hide(*hidden_output):
        env_path = '.'
        full_env_path = None

        # Check for virtualenv
        if create_env:
            env_path = '{}-env'.format(project_name)
            full_env_path = os.path.abspath(env_path)
            create_virtualenv(env_path)

        @virtualenv(full_env_path)
        def install_django():
            print(green('Installing Django...'))
            local('pip install "django>=1.4,<1.5"')

        @virtualenv(full_env_path)
        def create_project(project_name):
            print(green("Creating new Harvest project '{}'...".format(project_name)))
            local('django-admin.py startproject {} {}'.format(STARTPROJECT_ARGS, project_name))

        @virtualenv(full_env_path)
        def install_deps():
            print(green('Downloading and installing dependencies...'))
            local('pip install -r requirements.txt')

        @virtualenv(full_env_path)
        def collect_static():
            print(green('Collecting static files...'))
            local('make collect')

        @virtualenv(full_env_path)
        def syncdb(allow_input):
            print(green('Setting up a SQLite database...'))
            cmd = './bin/manage.py syncdb --migrate'
            if not allow_input:
                cmd += ' --noinput'
            local(cmd)

        install_django()

        create_project(project_name)

        # Change into project directory for next set of commands..
        os.chdir(project_name)

        # Ensure manage.py is executable..
        managepy_chmod()

        install_deps()

        collect_static()

    hidden_output = ['running']
    if not allow_input:
        hidden_output.append('stdout')

    # Refrain from blocking stdout due to the prompts..
    with hide(*hidden_output):
        syncdb(allow_input)

    print(green('\nComplete! Copy and paste the following in your shell:\n'))

    if create_env:
        print(green('cd {}/{}\nsource ../bin/activate'.format(env_path, project_name)))
    else:
        print(green('cd {}'.format(project_name)))

    print(green('./bin/manage.py runserver'))
    print(green('\nOpen up a web browser and go to: http://localhost:8000\n'))


parser.add_argument('project_name', help='Name of the Harvest project. This '
    'must be a valid Python identifier.')
parser.add_argument('-v', '--verbose', action='count',
    help='Print stdout output during installation process.')
parser.add_argument('--no-env', action='store_false', dest='create_env',
    help='Prevents creating a virtualenv and sets up the project in the '
        'current directory.')
parser.add_argument('--no-input', action='store_false', dest='allow_input',
    help='Prevents interactive prompts during setup.')



import os
from functools import wraps
from argparse import ArgumentParser
from fabric.context_managers import prefix


def cached_property(func):
    cach_attr = '_{}'.format(func.__name__)

    @property
    def wrap(self):
        if not hasattr(self, cach_attr):
            value = func(self)
            if value is not None:
                setattr(self, cach_attr, value)
        return getattr(self, cach_attr, None)
    return wrap


def cli(*args, **kwargs):
    def decorator(func):
        class Parser(ArgumentParser):
            def handle(self, *args, **kwargs):
                try:
                    func(*args, **kwargs)
                except Exception, e:
                    self.error(e.message)

            # No catching of exceptions
            def handle_raw(self, *args, **kwargs):
                func(*args, **kwargs)

        return Parser(*args, **kwargs)
    return decorator


def virtualenv(path):
    "Wraps a function and prefixes the call with the virtualenv active."
    if path is None:
        activate = None
    else:
        activate = os.path.join(path, 'bin/activate')

    def decorator(func):
        @wraps(func)
        def inner(*args, **kwargs):
            if path is not None:
                with prefix('source {}'.format(activate)):
                    func(*args, **kwargs)
            else:
                func(*args, **kwargs)
        return inner
    return decorator



import os
import sys
import stat
from fabric.api import local
from fabric.colors import red, green


def create_virtualenv(env_path):
    if os.path.exists(env_path):
        print(red("Error: Cannot create environment '{}'. A " \
            "directory with the same already exists.".format(env_path)))
        sys.exit()
    print(green("Setting up a virtual environment '{}'...".format(env_path)))
    local('virtualenv {}'.format(env_path))
    os.chdir(env_path)


def managepy_chmod():
    mode = stat.S_IMODE(os.stat('bin/manage.py').st_mode)
    executable = stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
    os.chmod('bin/manage.py', mode | executable)



from __future__ import print_function
import os
import urllib2
from zipfile import ZipFile
from fabric.api import local, hide
from fabric.colors import green
from harvest.decorators import cli, virtualenv
from harvest.utils import create_virtualenv, managepy_chmod

__doc__ = """\
Downloads and sets up a Harvest demo locally.
"""


DEMOS = {
    'openmrs': {
        'url': 'https://github.com/cbmi/harvest-openmrs/zipball/demo'
    }
}


def download_demo(demo_name):
    "Downloads and extracts the demo zip file"
    print(green("Downloading and unpacking '{}' demo...".format(demo_name)))
    response = urllib2.urlopen(DEMOS[demo_name]['url'])

    # Extract real name of zipfile
    content_disposition = response.headers.getheader('content-disposition')
    real_name = os.path.splitext(content_disposition.split('; ')[1].split('=')[1])[0]

    fname = '{}.zip'.format(demo_name)

    # Download zipfile
    with open(fname, 'wb') as tmpfile:
        while True:
            packet = response.read()
            if not packet:
                break
            tmpfile.write(packet)
        response.close()

    # Extract zipfile
    zipfile = ZipFile(fname)
    zipfile.extractall()

    # Clean up and rename to the correct demo name
    os.remove(fname)
    os.rename(real_name, demo_name)


@cli(description=__doc__)
def parser(options):
    demo_name = options.demo_name
    create_env = options.create_env
    verbose = options.verbose

    hidden_output = []

    if verbose < 1:
        hidden_output.append('stdout')
    if verbose < 2:
        hidden_output.append('running')

    with hide(*hidden_output):
        env_path = '.'
        full_env_path = None

        # Check for virtualenv
        if create_env:
            env_path = '{}-env'.format(demo_name)
            full_env_path = os.path.abspath(env_path)
            create_virtualenv(env_path)

        @virtualenv(full_env_path)
        def install_deps():
            print(green('Downloading and installing dependencies...'))
            local('pip install -r requirements.txt')

        @virtualenv(full_env_path)
        def collect_static():
            print(green('Collecting static files...'))
            local('make collect')

        @virtualenv(full_env_path)
        def syncdb(allow_input):
            print(green('Setting up a SQLite database...'))
            cmd = './bin/manage.py syncdb --migrate'
            if not allow_input:
                cmd += ' --noinput'
            local(cmd)

        # Download the demo
        download_demo(demo_name)

        # Change into project directory for next set of commands..
        os.chdir(demo_name)

        # Ensure manage.py is executable..
        managepy_chmod()

        install_deps()

        collect_static()

    print(green('\nComplete! Copy and paste the following in your shell:\n'))

    if create_env:
        print(green('cd {}/{}\nsource ../bin/activate'.format(env_path, demo_name)))
    else:
        print(green('cd {}'.format(demo_name)))

    print(green('./bin/manage.py runserver'))
    print(green('\nOpen up a web browser and go to: http://localhost:8000\n'))


parser.add_argument('demo_name', choices=sorted(DEMOS.keys()),
    help='The name of one of the Harvest project demos.')
parser.add_argument('-v', '--verbose', action='count',
    help='Print stdout output during installation process.')
parser.add_argument('--no-env', action='store_false', dest='create_env',
    help='Prevents creating a virtualenv and sets up the project in the '
        'current directory.')
